require "spec_helper"

describe Parsec do
  context "pos" do
    context "SourceName" do
      it "should have String as an ancestor" do
        SourceName("").class.ancestors.should include String
      end
    end

    context "Line" do
      it "should have Integer as an ancestor" do
        Line(1).class.ancestors.should include Integer
      end
    end

    context "Column" do
      it "should have Integer as an ancestor" do
        Column(1).class.ancestors.should include Integer
      end
    end

    context "SourcePos" do
      context "[]" do
        subject { SourcePos("a", 1, 1) }
        its([0]) { should eq "a" }
        its([1]) { should eq 1 }
        its([2]) { should eq 1 }
      end

      context ".to_s" do
        it { SourcePos("a", 1, 1).to_s.should eq '"a" (line 1, column 1)'}
        it { SourcePos(nil, 1, 1).to_s.should eq '(line 1, column 1)'}
      end
    end

    context "newPos" do
      it "should delegate to SourcePos" do
        a = newPos("", 1, 1)
        b = SourcePos("", 1, 1)
        a[0].should eq b[0]
        a[1].should eq b[1]
        a[2].should eq b[2]
      end
    end

    context "initialPos" do
      subject { initialPos "" }

      its([0]) { should eq "" }
      its([1]) { should eq 1 }
      its([2]) { should eq 1 }
    end

    context "sourceName, sourceLine, sourceColumn" do
      subject { SourcePos("", 1, 1) }
      it { sourceName(subject).should eq "" }
      it { sourceLine(subject).should eq 1 }
      it { sourceColumn(subject).should eq 1 }
    end

    context "incSourceLine, incSourceColumn" do
      subject { SourcePos("", 1, 2) }
      it { sourceLine(incSourceLine(subject, 2)).should eq 3 }
      it { sourceColumn(incSourceColumn(subject, 3)).should eq 5 }
    end

    context "setSourceName, setSourceLine, setSourceColumn" do
      subject { SourcePos("", 1, 1) }
      it { sourceName(setSourceName(subject, "a")).should eq "a" }
      it { sourceLine(setSourceLine(subject, 2)).should eq 2 }
      it { sourceColumn(setSourceColumn(subject, 3)).should eq 3 }
    end

    context "updatePosString" do
      subject { updatePosString(SourcePos("", 1, 1), "a\nb\t\tc") }
      it { sourceLine(subject).should eq 2 }
      it { sourceColumn(subject).should eq 18 }
    end

    context "updatePosChar" do
      subject { updatePosChar(SourcePos("", 1, 1), "a") }
      it { sourceLine(subject).should eq 1 }
      it { sourceColumn(subject).should eq 2 }
    end
  end
end
